@@ -3,13 +3,14 @@ from enum import Enum
 from os import PathLike
 from pathlib import PurePosixPath
 from textwrap import indent
+from typing import Dict, List
 
 INDENTATION = '  '
 
 
-class ModelType(Enum):
+class EntryType(Enum):
     """
-    abbreviated model type within a NEMS / NUOPC configuration file
+    possible types of a configuration entry
     """
 
     ATMOSPHERIC = 'ATM'
@@ -20,9 +21,9 @@ class ModelType(Enum):
     MEDIATOR = 'MED'
 
 
-class ModelVerbosity(Enum):
+class VerbosityOption(Enum):
     """
-    verbosity attribute within a NEMS / NUOPC configuration file
+    possible verbosity options for model entries
     """
 
     OFF = 'off'
@@ -31,9 +32,9 @@ class ModelVerbosity(Enum):
     MAX = 'max'
 
 
-class RemapMethod(Enum):
+class GridRemapMethod(Enum):
     """
-    model remapping methods
+    possible remapping methods (interpolation / redistribution) for translating coupled data from one grid to another
     """
 
     REDISTRIBUTE = 'redist'
@@ -44,15 +45,28 @@ class RemapMethod(Enum):
     CONSERVE = 'conserve'
 
 
-class ModelMeshEntry(ABC):
-    def __init__(self, mesh_type: ModelType, filename: PathLike = None):
+class FileForcingEntry(ABC):
+    """
+    abstraction of a forcing entry in ``config.rc``, defining the file path to a forcing file
+    """
+
+    def __init__(self, entry_type: EntryType, filename: PathLike = None):
+        """
+        :param entry_type: type of file forcing (i.e. ``ATM``, ``ICE``, etc.)
+        :param filename: path to file
+        """
+
         if filename is not None and not isinstance(filename, PurePosixPath):
             filename = PurePosixPath(filename)
 
-        self.mesh_type = mesh_type
+        self.mesh_type = entry_type
         self.filename = filename
 
     def __str__(self) -> str:
+        """
+        string representation of the forcing entry in ``config.rc``
+        """
+
         if self.filename is not None:
             directory = self.filename.parent.as_posix()
             name = self.filename.name
@@ -68,18 +82,41 @@ class ModelMeshEntry(ABC):
 
 class ConfigurationEntry(ABC):
     """
-    NEMS / NUOPC configuration entry within `nems.configure`
+    abstraction of an entry in a configuration file
     """
 
-    entry_type: str = NotImplementedError
-    __attributes: {str: str} = NotImplementedError
+    @classmethod
+    @abstractmethod
+    def from_string(cls, string: str, **kwargs) -> 'ConfigurationEntry':
+        """
+        parse entry information from a configuration entry string
+        """
+
+        raise NotImplementedError
+
+
+class AttributeEntry(ABC):
+    """
+    abstraction of a configuration entry in ``nems.configure``
+    """
+
+    entry_title: str = NotImplementedError
+    __attributes: Dict[str, str] = NotImplementedError
 
     @abstractmethod
     def __str__(self) -> str:
+        """
+        string representation of the configuration entry in ``nems.configure``
+        """
+
         raise NotImplementedError
 
     @property
-    def attributes(self) -> {str: str}:
+    def attributes(self) -> Dict[str, str]:
+        """
+        attributes defining options for the configuration entry, such as verbosity
+        """
+
         attributes = {}
         for attribute, value in self.__attributes.items():
             if isinstance(value, Enum):
@@ -90,33 +127,44 @@ class ConfigurationEntry(ABC):
         return attributes
 
     @attributes.setter
-    def attributes(self, attributes: {str: str}):
+    def attributes(self, attributes: Dict[str, str]):
         self.__attributes = attributes
 
 
 class SequenceEntry(ABC):
     """
-    entry within run sequence
+    abstraction of an entry within the run sequence in ``nems.configure``
     """
 
     @property
     @abstractmethod
     def sequence_entry(self) -> str:
+        """
+        string representation of the sequence entry in ``nems.configure``
+        """
+
         raise NotImplementedError
 
     def __str__(self) -> str:
         return self.sequence_entry
 
 
-class ModelEntry(ConfigurationEntry, SequenceEntry):
+class ModelEntry(AttributeEntry, SequenceEntry, ConfigurationEntry):
     """
-    abstract implementation of a generic model
+    abstraction of a generic model implementing NEMS / NUOPC coupling
     """
 
-    model_type: ModelType
+    entry_type: EntryType
     name: str
 
     def __init__(self, processors: int, **attributes):
+        """
+        The model entry is represented in two places in ``nems.configure``: once as a configuration entry with attributes, and once in the run sequence.
+        The specific processors assigned to the model are defined by start and stop indices, which are determined on configuration write from the stop index of the previous entry in the run sequence.
+
+        :param processors: number of processors to assign to this model
+        """
+
         self.__processors = processors
 
         self.__start_processor = None
@@ -125,21 +173,29 @@ class ModelEntry(ConfigurationEntry, SequenceEntry):
         self.__next = None
 
         if 'Verbosity' not in attributes:
-            attributes['Verbosity'] = ModelVerbosity.OFF
+            attributes['Verbosity'] = VerbosityOption.OFF
 
         # http://www.earthsystemmodeling.org/esmf_releases/last_built/NUOPC_refdoc/node3.html#SECTION00033000000000000000
         self.attributes = attributes
 
     @property
-    def entry_type(self) -> str:
-        return str(self.model_type.value) if self.model_type is not None else None
+    def entry_title(self) -> str:
+        return str(self.entry_type.value) if self.entry_type is not None else None
 
     @property
     def processors(self) -> int:
+        """
+        the number of processors assigned to this model
+        """
+
         return self.__processors
 
     @processors.setter
     def processors(self, processors: int):
+        """
+        set the number of processors (this also updates all subsequent entries in the run sequence)
+        """
+
         if processors != self.processors:
             self.__processors = processors
             # update following processors
@@ -147,10 +203,18 @@ class ModelEntry(ConfigurationEntry, SequenceEntry):
 
     @property
     def start_processor(self) -> int:
+        """
+        the first index in the processor series assigned to this model
+        """
+
         return self.__start_processor
 
     @start_processor.setter
     def start_processor(self, index: int):
+        """
+        set the first index in the processor series (this also updates all subsequent entries in the run sequence)
+        """
+
         self.__start_processor = index
         current_model = self.next
         while current_model is not None:
@@ -160,6 +224,10 @@ class ModelEntry(ConfigurationEntry, SequenceEntry):
 
     @property
     def end_processor(self) -> int:
+        """
+        the last index in the processor series assigned to this model
+        """
+
         if self.start_processor is not None:
             return self.start_processor + self.processors - 1
         else:
@@ -167,10 +235,18 @@ class ModelEntry(ConfigurationEntry, SequenceEntry):
 
     @property
     def previous(self) -> 'ModelEntry':
+        """
+        the previous entry in the run sequence
+        """
+
         return self.__previous
 
     @previous.setter
     def previous(self, previous: 'ModelEntry'):
+        """
+        assign the previous entry in the run sequence (this also updates the processor series for this and all subsequent entries in the run sequence)
+        """
+
         if previous is None and self.previous is not None:
             self.previous.__next = None
         self.__previous = previous
@@ -183,10 +259,18 @@ class ModelEntry(ConfigurationEntry, SequenceEntry):
 
     @property
     def next(self) -> 'ModelEntry':
+        """
+        the next entry in the run sequence
+        """
+
         return self.__next
 
     @next.setter
     def next(self, next: 'ModelEntry'):
+        """
+        assign the next entry in the run sequence (this updates all subsequent entries in the run sequence)
+        """
+
         if next is None and self.next is not None:
             self.next.__previous = None
         self.__next = next
@@ -195,14 +279,14 @@ class ModelEntry(ConfigurationEntry, SequenceEntry):
 
     @property
     def sequence_entry(self) -> str:
-        return str(self.model_type.value)
+        return str(self.entry_type.value)
 
     def __str__(self) -> str:
         return '\n'.join(
             [
-                f'{self.entry_type}_model:                      {self.name}',
-                f'{self.entry_type}_petlist_bounds:             {self.start_processor} {self.end_processor}',
-                f'{self.entry_type}_attributes::',
+                f'{self.entry_type.value}_model:                      {self.name}',
+                f'{self.entry_type.value}_petlist_bounds:             {self.start_processor} {self.end_processor}',
+                f'{self.entry_type.value}_attributes::',
                 indent(
                     '\n'.join(
                         [
@@ -218,17 +302,17 @@ class ModelEntry(ConfigurationEntry, SequenceEntry):
 
     def __repr__(self) -> str:
         kwargs = [f'{key}={value}' for key, value in self.attributes.items()]
-        return f'{self.__class__.__name__}({repr(self.name)}, {self.model_type}, {self.processors}, {", ".join(kwargs)})'
+        return f'{self.__class__.__name__}({repr(self.name)}, {self.entry_type}, {self.processors}, {", ".join(kwargs)})'
 
     @classmethod
     def from_string(cls, string: str, **kwargs) -> 'ModelEntry':
         lines = string.splitlines()
 
         parsed_model_type, parsed_name = (value.strip() for value in lines[0].split('_model:'))
-        parsed_model_type = ModelType(parsed_model_type)
+        parsed_model_type = EntryType(parsed_model_type)
 
         if hasattr(cls, 'model_type'):
-            assert parsed_model_type == cls.model_type
+            assert parsed_model_type == cls.entry_type
         if hasattr(cls, 'name'):
             assert parsed_name == cls.name
 
@@ -244,27 +328,41 @@ class ModelEntry(ConfigurationEntry, SequenceEntry):
         instance = cls(processors=end_processor + 1 - start_processor, **attributes, **kwargs,)
 
         if not hasattr(cls, 'model_type'):
-            instance.model_type = parsed_model_type
+            instance.entry_type = parsed_model_type
         if not hasattr(cls, 'name'):
             instance.name = parsed_name
 
         return instance
 
 
-class ConnectionEntry(SequenceEntry):
-    def __init__(self, source: ModelEntry, target: ModelEntry, method: RemapMethod = None):
+class ConnectionEntry(SequenceEntry, ConfigurationEntry):
+    """
+    a connection entry in ``nems.configure`` representing a simple coupling between two model entries
+    """
+
+    def __init__(self, source: ModelEntry, target: ModelEntry, method: GridRemapMethod = None):
+        """
+        :param source: source model entry
+        :param target: target model entry
+        :param method: remapping method with which to translate between differing grids (use ``redist`` for the same grid)
+        """
+
         self.source = source
         self.target = target
-        self.method = method if method is not None else RemapMethod.BILINEAR
+        self.method = method if method is not None else GridRemapMethod.BILINEAR
 
     @property
-    def models(self) -> [ModelEntry]:
+    def models(self) -> List[ModelEntry]:
+        """
+        the source and target models in the coupling
+        """
+
         return [self.source, self.target]
 
     @property
     def sequence_entry(self) -> str:
         return (
-            f'{self.source.entry_type} -> {self.target.entry_type}'.ljust(13)
+            f'{self.source.entry_type.value} -> {self.target.entry_type.value}'.ljust(13)
             + f':remapMethod={self.method.value}'
         )
 
@@ -272,7 +370,7 @@ class ConnectionEntry(SequenceEntry):
         return f'{self.__class__.__name__}({repr(self.source)}, {repr(self.target)}, {repr(self.method)})'
 
     @classmethod
-    def from_string(cls, string: str) -> 'ConnectionEntry':
+    def from_string(cls, string: str, **kwargs) -> 'ConnectionEntry':
         method = None
         try:
             source, target = (entry.strip() for entry in string.split('->', 1))
@@ -284,56 +382,91 @@ class ConnectionEntry(SequenceEntry):
         if ':' in target:
             target, parsed_method = (entry.strip() for entry in target.split(':', 1))
             if method is None and len(parsed_method) > 0:
-                method = RemapMethod(parsed_method.split('=')[-1])
+                method = GridRemapMethod(parsed_method.split('=')[-1])
 
         source_model = ModelEntry(None)
         target_model = ModelEntry(None)
 
         source_model.name = source
-        source_model.model_type = ModelType(source)
+        source_model.entry_type = EntryType(source)
         target_model.name = target
-        target_model.model_type = ModelType(target)
+        target_model.entry_type = EntryType(target)
 
         return cls(source=source_model, target=target_model, method=method)
 
 
 class MediatorEntry(ModelEntry):
-    model_type = ModelType.MEDIATOR
+    """
+    a special entry in ``nems.configure`` representing a coupler between two model entries with a dedicated coupling function
+    """
+
+    entry_type = EntryType.MEDIATOR
     name = 'implicit'
 
     def __init__(self, processors: int = None, **attributes):
+        """
+        :param processors: number of processors to assign to this mediator
+        """
+
         if processors is None:
             processors = 1
         super().__init__(processors, **attributes)
 
 
-class MediationFunctionEntry(SequenceEntry):
+class MediationFunctionEntry(SequenceEntry, ConfigurationEntry):
+    """
+    the dedicated function of a mediation entry, applied to the coupling between two model entries in ``nems.configure``
+    """
+
     def __init__(self, name: str, mediator: MediatorEntry):
+        """
+        :param name: name of function
+        :param mediator: mediator entry to host this function
+        """
+
         self.name = name
         self.mediator = mediator
 
     @property
     def sequence_entry(self) -> str:
-        return f'{self.mediator.model_type.value} {self.name}'
+        return f'{self.mediator.entry_type.value} {self.name}'
 
     def __repr__(self) -> str:
         return f'{self.__class__.__name__}({repr(self.name)}, {repr(self.mediator)})'
 
     @classmethod
-    def from_string(cls, string: str) -> 'MediationFunctionEntry':
+    def from_string(cls, string: str, **kwargs) -> 'MediationFunctionEntry':
+        """
+        parse entry information from a configuration entry string
+        """
+
         mediator_name, function_name = string.split()
-        return cls(name=function_name, mediator=MediatorEntry(mediator_name),)
+        return cls(name=function_name, mediator=MediatorEntry(mediator_name))
 
 
-class MediationEntry(SequenceEntry):
+class MediationEntry(SequenceEntry, ConfigurationEntry):
+    """
+    an application of a mediator between model entries, with a dedicated coupling function
+    """
+
     def __init__(
         self,
         mediator: MediatorEntry,
-        sources: [ModelEntry] = None,
-        functions: [str] = None,
-        targets: [ModelEntry] = None,
-        method: RemapMethod = None,
+        sources: List[ModelEntry] = None,
+        functions: List[str] = None,
+        targets: List[ModelEntry] = None,
+        method: GridRemapMethod = None,
     ):
+        """
+        A Mediation can include multiple connections; for instance, an ICE model might connect to the mediation function, then the mediation function connects to the OCN model.
+
+        :param mediator: mediator entry to host this mediation
+        :param sources: source models
+        :param functions: mediation functions to use
+        :param targets: target models
+        :param method: remapping method with which to translate between differing grids (use ``redist`` for the same grid)
+        """
+
         if functions is None:
             functions = []
 
@@ -348,7 +481,11 @@ class MediationEntry(SequenceEntry):
         self.method = method
 
     @property
-    def source_connections(self) -> [ConnectionEntry]:
+    def source_connections(self) -> List[ConnectionEntry]:
+        """
+        list of connections between the source(s) and the mediator
+        """
+
         source_connections = []
         if self.sources is not None:
             for index, source in enumerate(self.sources):
@@ -356,7 +493,11 @@ class MediationEntry(SequenceEntry):
         return source_connections
 
     @property
-    def target_connections(self) -> [ConnectionEntry]:
+    def target_connections(self) -> List[ConnectionEntry]:
+        """
+        list of connections between the mediator and the target(s)
+        """
+
         target_connections = []
         if self.targets is not None:
             for index, target in enumerate(self.targets):
@@ -364,7 +505,11 @@ class MediationEntry(SequenceEntry):
         return target_connections
 
     @property
-    def models(self) -> [ModelEntry]:
+    def models(self) -> List[ModelEntry]:
+        """
+        list of model entries involved in the mediation
+        """
+
         return [
             *(connection.source for connection in self.source_connections),
             self.mediator,
@@ -391,7 +536,7 @@ class MediationEntry(SequenceEntry):
         return f'{self.__class__.__name__}({repr(self.mediator)}, {repr(self.sources)}, {repr(self.functions)}, {repr(self.targets)}, {repr(self.method)})'
 
     @classmethod
-    def from_string(cls, string: str) -> 'MediationEntry':
+    def from_string(cls, string: str, **kwargs) -> 'MediationEntry':
         lines = string.strip().splitlines()
 
         connections = []
